#ifndef __CPM_H__
#define __CPM_H__

#include <stdint.h>

/* Public functions. */

void cpm_init();

int jz4740_cpm_have_pll(void *cpm_base);

int jz4740_cpm_have_clock(void *cpm_base);
void jz4740_cpm_start_clock(void *cpm_base);

uint32_t jz4740_cpm_get_cpu_frequency(void *cpm_base);
uint32_t jz4740_cpm_get_memory_frequency(void *cpm_base);

void jz4740_cpm_set_lcd_frequencies(void *cpm_base, uint32_t pclk, uint8_t ratio);
void jz4740_cpm_update_output_frequency(void *cpm_base);

void jz4740_cpm_start_lcd(void *cpm_base);
void jz4740_cpm_stop_lcd(void *cpm_base);

uint32_t jz4740_cpm_ctrl_get(void *cpm_base, uint32_t reg);
void jz4740_cpm_ctrl_set(void *cpm_base, uint32_t reg, uint32_t value);

/* Register offsets. */

#define CPM_CPCCR	0x00
#define CPM_LCR		0x04
#define CPM_RSR		0x08
#define CPM_CPPCR	0x10

#ifdef CONFIG_CPU_JZ4730

/* Names used by the jz4730. */

#define CPM_CFCR	CPM_CPCCR
#define CPM_LPCR	CPM_LCR
#define CPM_RSTR	CPM_RSR
#define CPM_PLCR1	CPM_CPPCR
#define CPM_CFCR2	CPM_I2SCDR	/* apparently used by the LCD */

/* Registers used by the jz4730. */

#define CPM_OCR		0x1c
#define CPM_MSCR	0x20		/* different layout to CLKGR */
#define CPM_WRER	0x28
#define CPM_WFER	0x2c
#define CPM_WER		0x30
#define CPM_WSR		0x34
#define CPM_GSR0	0x38
#define CPM_GSR1	0x3c
#define CPM_GSR2	0x40
#define CPM_SPR		0x44
#define CPM_GSR3	0x48

#else

/* Registers used by the jz4740. */

#define CPM_CLKGR	0x20
#define CPM_SCR		0x24
#define CPM_HCR		0x30
#define CPM_HWFCR	0x34
#define CPM_HRCR	0x38
#define CPM_HWCR	0x3c
#define CPM_HWSR	0x40
#define CPM_HSPR	0x44
#define CPM_I2SCDR	0x60
#define CPM_LPCDR	0x64
#define CPM_MSCCDR	0x68
#define CPM_UHCCDR	0x6C

#endif

#endif /* __CPM_H__ */
